// MODEL 203 DATA ACQUISITON SYSTEM 

// Lawson Labs, Inc.
// 3239 Phoenixville Pike
// Malvern, PA  19355
// 800 321-5355 or 610 725-8800
// December 18, 1998

//*************************************************************************//
//  INCLUDE FILES 
//
//#define STRICT
#include <windows.h>
#include <stdio.h>
#include <dos.h>
#include <stdlib.h>
#include <conio.h>
#include <math.h>
#include <ctype.h>
#include <string.h>
#include <time.h>
#include "VC5_m203.h"
#include <winbase.h>

//
//*************************************************************************//
//  LOCAL INTERNAL DEFINES

#define NO_DEBUG_FILE 0
#define SIMPLE_DEBUG_FILE 1
#define NO_ECHO_TEST 0
#define DO_ECHO_TEST 1
#define DAC_1 1
#define DAC_2 2

HANDLE	hOutCon, hInCon;
HMODULE hLibM203;
COORD	pos, dwCursorPosition;
CONSOLE_CURSOR_INFO	ConsoleCursorInfo;
CONSOLE_SCREEN_BUFFER_INFO lpConsoleScreenBufferInfo;
DWORD	lpNumberOfCharsWritten;

// following macro allows use of gotoxy and clrscr 
// functions used by previous 16-bit and Win32s code
#define gotoxy(x, y) pos.X = x; pos.Y = y - 1;	\
SetConsoleCursorPosition(hOutCon, pos);
#define clrscr() pos.X = 0; pos.Y = 0;		\
FillConsoleOutputCharacter(hOutCon, ' ',	\
80*25, pos, &lpNumberOfCharsWritten);

T_M203  M203;  
T_M203 *pM203;


//********************************************************************/
// MAIN	- This routine consists of three major sections; TOP_MENU,
//		POLLED, and GET_SCANS. 
//
//	 TOP_MENU performs system initialization, and provides 
//		the top menu functions; Serial Port Selection, BAUD rate selection
//		Input buffer size adjustment, and a Serial Port echo test. 
//
//	 POLLED establishes serial communications with the Model 203 
//		(Sign On) and performs data acquisition in polled mode.  Command 
//		provides a means to configure the Model 203 Data Aquisition parameters
//		such as Data Rate, Local Averaging, etc.
//	  	
//	 GET_SCANS controls scanning parameter selection, data logging,
//		and scanning mode operation.
//
/*********************************************************************/
int main(void)
{
	BOOLEAN fScanRunning = FALSE;
	UINT uiDbgLoopCnt = 0;

	short sRetVal; long lNumScansInBuff = 0;
	pM203 = &M203;
	pAppNewScanBuff = AppNewScanBuff;

	long	scanCnt;  		// Scanning counter - Incremented for each scan 
	int		printRow;  		// Polled Mode display row address
	int		keyinput;		// Last keyboard input character
	char	tmpbuf[80];		// Temporary string array 

	
	hOutCon = GetStdHandle(STD_OUTPUT_HANDLE);
	if(!hOutCon)
	{
		MessageBox(0, "Error writing to console", "-- error", MB_OK);
		return 1;
	}
	hInCon =  GetStdHandle(STD_INPUT_HANDLE);
	if(!hInCon)
	{
		MessageBox(0, "Error reading from console", "-- error", MB_OK);
		return 1;
	}


	ConsoleCursorInfo.dwSize=1;
	ConsoleCursorInfo.bVisible = TRUE;


	if(!LoadLibraries())
	{
		exit(1);
	}

	if(!CheckLibFuncs())
	{
		UnloadLibraries();
		exit(1);
	}


	SetWin32CursorOff();   		// Disable Windows cursor
	M203.digOutWord=256;		// Initialize digOutWord to "Not Sent" state
	if(ReadConfig()==1)			// Read configuration file (if present)
	{
		gotoxy(18,10); printf("unable to read M203.CFG configuration file"); 
		gotoxy(26,11); printf("press any key to continue");
		getch();
	}

	if(!SetPtrs(pM203, pAppNewScanBuff, BuffDebugMsg))
	{
		MessageBox(0, "\nCould not initialize memory pointers\n\n"
					"shared by application and DLL", "CRITICAL ERROR ---", 0);
	}

TOP_MENU:
	PrintTopMenuScreen();

TOP_MENU_LOOP:
	keyinput = getch();
	switch (keyinput)	// which sign on parameter to change
	{
		case ('1'):		// Select BAUD rate			
			if (M203.baud++ == 5)M203.baud = 0;
			gotoxy(47,7); printf ("%4d",bauds[M203.baud]);
			break;
		case ('2'):		// Select COM port
			M203.comPort++;
			if (M203.comPort == 4){
				M203.comPort = 0;
			}
			gotoxy(47,9);
			printf ("COM%1d",M203.comPort+1);
			break;
		case ('3'):  	// Adjust Input Buffer Size
			gotoxy (22,24); printf("Enter buffer size (11 to 32767 bytes):");
			SetWin32CursorOn();
			gotoxy (66,24);
			getch();
			fgets(tmpbuf,80,stdin);
			sscanf(tmpbuf,"%d", M203.inputBuffer);
			if (M203.inputBuffer < 11) M203.inputBuffer = 11;
			if (M203.inputBuffer > 32767) M203.inputBuffer = 32767;
			SetWin32CursorOff();
			gotoxy(46,11);	
			printf ("%6d bytes   ",M203.inputBuffer);
			ClearCommandLines(); 
			break;
		case ('4'): 	// Perform Serial Port Test
			M203.echoTestCount = 0;
			if(SignOn(DO_ECHO_TEST, fDebug)!=0)
			{
				gotoxy(3,24); printf("echo test error   Press any key to continue");
				getch();
				goto TOP_MENU_LOOP;
			}
			gotoxy(3,24); printf("Serial Echo Test  Press any key to continue");
			while(kbhit()==0) 			//loop until keyboard hit
			{
				if (EchoTest()==0)
				{
					gotoxy(50,24);	printf ("bytes echoed : %6d ",M203.echoTestCount);
				}
				else
				{
					gotoxy(3,24); printf("echo test error   Press any key to continue");
				}
			}
			getch();					// Get and through away keyboard input
			if(SignOn(NO_ECHO_TEST, fDebug)!=0)     // Finish Signon
			{
				gotoxy(3,24); printf("SignOn Error  Press any key to continue");
				getch();
				goto TOP_MENU_LOOP;
			}
			ClearCommandLines(); 
			break;

		case ('5'):		// Select COM port
			gotoxy(47,15);
			
			fDebug++;
			if (fDebug == 0){
				printf("none  ");
			}
			if (fDebug == 1){
				printf("low   ");
			}
			if (fDebug == 2){
				printf("high  ");
			}
			if (fDebug == 3){
				fDebug = 0;
				printf("none  ");
			}
			
			break;
		case 27:				// ESC exit
			goto EXIT_POINT;

		case 13: 				// Enter Start Polled Data Acquisition
			M203.dataAcqMode=1;
			goto POLLED;

		default:
			goto TOP_MENU_LOOP;	

	} // end of switch
	goto TOP_MENU_LOOP;

//********************************************************************/
// POLLED DATA AQUISITION
//********************************************************************/
POLLED:          	
	clrscr(); 
	gotoxy(30,12); printf("Signing on...");
	if (SignOn(NO_ECHO_TEST, fDebug)==1)
	{
		gotoxy(16,14);	printf ("unable to sign on   Press any key to continue");
		getch();
		goto TOP_MENU;          
	}
	printf (" Successful in %2d Trys",(M203.retry+1));
	Wait(1000);
	clrscr();
	if (GetVer()!=0)			// Get Model 203 Version
	{
		gotoxy(15,12);	printf("unable to read version  press any key to continue.");
		getch();
	}

	if (GetCheckSum()!=0)		// Test Model 203 CheckSum
	{
		gotoxy(15,12);	printf("unable to read checksum   press any key to continue.");
		getch();
	}

	ChangeAnalogOutput(DAC_2);  // Initialize Analog Outputs


	ChangeAnalogOutput(DAC_2);  // config file or default values

	gotoxy(25,12);	printf("performing system calibration...");
	ClearVoltageArray(); 		// Clear and reset running average array
	SysCal();					// Perform a DAS system calibration 
POLLED1:
	ClearVoltageArray();		// Clear running average array again
	ClearSD();               		// Clear standard deviation calculation array
	PrintPolledScreen();	  	// Draw polled screen
	printRow=14;
POLLED2:						// POLLED Mode "main executive" loop
	SetWin32CursorOff();
	while (kbhit()==0) 			//loop until keyboard hit
	{
		if (GetVoltage_LOCAL(&Voltage)!=0)	// Get Next Voltage 
		{
			gotoxy(1,22);
			printf(BuffDebugMsg);
			printf("\nget conversion failed press any key to continue    \n");
			getch();
			goto TOP_MENU;  
		}
		if(Max < Voltage) Max = Voltage;	// update maximum
		if(Min > Voltage) Min = Voltage;	// update minimum
		StandardDev(Voltage, &stDv, &rmsBits, &sdTot, sd, &Average, &sdCount);
		gotoxy(4,printRow);                
		printf ("%9ld    %9.3lf",M203.rawCount,Voltage);
		gotoxy(4,printRow); printf (" ");  // Erase the previous "*" marker
		if (printRow++ >= 22) // Update Standard Deviation Display
		{
			printRow = 14;
			gotoxy (40,14);
			printf ("Maximum             %9.3lf mv",Max);
			gotoxy (40,15);
			printf ("Minimum             %9.3lf mv",Min);
			gotoxy (40,16);
			printf ("Delta               %9.3lf mv",fabs(Max - Min));
			gotoxy (40,17);
			printf ("Average             %9.3lf mv",Average);
			gotoxy (40,19);
			printf ("Standard Deviation  %9.3lf mv",stDv);
			gotoxy (40,20);
			printf ("Maximum Resolution     %3.1lf bits",M203.totalBits);
			gotoxy (40,21);
			printf ("Effective Resolution   %3.1lf bits",rmsBits);
			gotoxy (1,14);
		} // end of if(printRow >= 22)
		gotoxy(4,printRow); printf ("*"); // update location of latest reading marker
	}   //end of while !kbhit()
	keyinput = getch();
	if (islower(keyinput)) keyinput = toupper(keyinput);
	if (keyinput == 'N')			// Test for "N" to start scanning
	{
		M203.dataAcqMode=0;
		goto GET_SCANS;
	}
	if (keyinput == 27)				// Test for ESC character
	{
		goto TOP_MENU;
	}               
	else
	{
		DoCommand(keyinput);		// Process keyboard input command
		goto POLLED2;   
	}

//********************************************************************
//	GET_SCANS 
//********************************************************************
GET_SCANS:  
	// Set up scanning parameters
	if (ScanMenu()==1) goto POLLED1;  	//setup scanning details

	// Sign On with selected scan interval
	clrscr();
	gotoxy(30,12); printf("Signing on...");
	if (SignOn(NO_ECHO_TEST, fDebug)==1)
	{
		gotoxy(16,14);
		printf ("Unable to sign on. Press any key to continue.");
		getch();
		goto GET_SCANS;
	}

	// Reset analog outputs
	ChangeAnalogOutput(DAC_1);
	ChangeAnalogOutput(DAC_2);

	// Perform system Calibration
	clrscr();
	gotoxy(20,12); printf("Performing system calibration...");
	ClearVoltageArray();
	SysCal();
	ClearVoltageArray();
	ClearSD();


	// Start Scanning
	StartScanning(ScanToken,Edge);  	//Start scanning using selected scan type

	// Set Up Scan Display Screen
	clrscr();
	scanCnt = 1;
	gotoxy (3,2); 
	printf ("SCANNING             Lawson Labs, Inc.  Model 203");
	gotoxy (60,1);     
	printf ("Version = %3d\n",M203.version);
	gotoxy (5,4); 
	printf("SCAN      COUNT     MILLIVOLTS     CHANNEL : %1d",M203.scanChan);
	gotoxy(3,22);
	printf("Scan interval %10.3lf secs",M203.scanInterval);
	gotoxy(3,24);
	printf("Press any key to quit");
	printRow=5;

GET_SCANS1:
	SetWin32CursorOff();
	uiDbgLoopCnt=0;
	while(!kbhit())				//while no keystroke
	{
		if(!fScanRunning)
        if((ScanToken!=0x8B) && (ScanToken!=0x8D))	//not a triggered scan
		{
			while(!fScanRunning)
			{
				if(kbhit()) 
					goto END_SCANS;
				ScanError = GetVoltage(); //just used here to test for scan trigger 

				ClearCommandLines(); gotoxy (3,24); 
				if (ScanError == 0) { fScanRunning = TRUE; break; }		// if no errors
				//uiDbgLoopCnt++;

				switch (ScanError)
				{
					case -1:
						if (Edge == 0)
						{ printf("waiting for positive edge trigger..."); }
						//{ printf("waiting for positive edge trigger... cnt: %u",uiDbgLoopCnt); }
						else
						{ printf("waiting for negative edge trigger..."); }
						//{ printf("waiting for negative edge trigger... cnt: %u",uiDbgLoopCnt); }
						//getch();
						continue; //goto GET_SCANS1;
					case 1:
						printf("Time Out Waiting for next scan - press any key to continue ");
						goto END_SCANS; //break;
					case 2:
						printf("Input Buffer Overflow - press any key to continue ");
						goto END_SCANS; //break;
					default:
						printf("Invalid Error Message - press any key to continue ");
						goto END_SCANS; //break;
				}

				lNumScansInBuff = GetNumScansInBuffer();
				if(lNumScansInBuff) //keep buffer empty
				{ for(UINT uiNumInBuff = 0;uiNumInBuff < lNumScansInBuff; uiNumInBuff++)
					{ GetOneScan(); Gconv(); } }
			}
		}
		

		if(scanCnt==1)
		{ ClearCommandLines(); gotoxy(3,24); printf("Press any key to quit"); }

		fScanRunning = TRUE;
		lNumScansInBuff = GetNumScansInBuffer();
		if(!lNumScansInBuff) continue;
		
		for(UINT uiNumInBuff = 0;uiNumInBuff < lNumScansInBuff; uiNumInBuff++)
		{
			if(kbhit()) break;
			
			if(GetOneScan())
			{	
				gotoxy (5,6);
				printf("\n scanning error   Press any key to continue "); getch();
				goto END_SCANS;
			}
			Gconv();
			ScanError = 0;
			
			if (printRow == 6) 
			{
				gotoxy (5,6); 
				printf("                                         ");
			}
			gotoxy (3,printRow); printf ("%6d",scanCnt);
			gotoxy (11,printRow); printf("%9ld      %9.6f",M203.rawCount,M203.Voltage/1000);
			gotoxy(2,printRow); printf (" ");  // Erase the previous "*" marker
			if (fLogToFile==1)
			{
				if (LogFile!=0)
				{
					fprintf(LogFile,"%6d\t",scanCnt);
					fprintf(LogFile,"%9ld\t",M203.rawCount);
					fprintf(LogFile,"%9.3lf\n",M203.Voltage/1000);
				}
				else
				{ printf("Error: Log File is not valid"); }
			}
			scanCnt = scanCnt+1;
			if (printRow++ >= 20) printRow = 5;
			gotoxy(2,printRow); printf ("*"); // update lastest reading marker
		}
	} //end while !kbhit()
END_SCANS:  // Scanning is discontinued.
	getch();
	fScanRunning = FALSE;

	sRetVal = StopScanning();			//Tell the Model 203 to stop scanning
	ClearCommandLines(); gotoxy (3,24); 
	if(sRetVal) 
		printf("failed to end scan correctly - Press any key to continue ");
	else
		printf("successful scan end - Press any key to continue ");
	getch();

	if (fLogToFile==1)
	{
		if (fclose(LogFile)==0)
		{
			LogFile = 0;
			fLogToFile=0;
		}
		else
		{
			ClearCommandLines();
			gotoxy (3,24); printf("Unable to close Log File   Press any key to continue");
			getch();
		}
	}

	goto GET_SCANS;     		   //end of scanning example

EXIT_POINT:
	clrscr();
	gotoxy(15,12); printf("Do you wish to save configuration (y/n)?  ");
	keyinput = getch();
	if (keyinput == 'Y' || keyinput == 'y')
	{
		WriteConfig(); 	// Save current operation parameters in configuration file
	}
	clrscr();
	gotoxy(15,12); printf("Do you wish to exit program (y/n)?  ");
	keyinput = getch();
	if((keyinput != 'Y')&&(keyinput != 'y'))
	{
		goto TOP_MENU;
	}
	DestroyWindow(GetActiveWindow());
	exit(0);
	return 1;
}// end of Main Function
 
//********************************************************************/
// LOCAL FUNCTIONS 
//********************************************************************/
// STANDARD DEVIATION - This function calculates the voltage average,
//		standard deviation, and RMS noise in bits of a group of 20 voltage
//		readings.
//
// Input: Voltage = latest voltage reading, M203.totalBits = maximum resolution
//		in bits under current conditions (sample rate, running average, etc.)	
// Other Functions Called: None
// Output: (every 20 readings) stDv = Standard Deviation, Average = average
//		of 20 readings, rmsBits = RMS noise in Bits for 20 readings
// Output - Return Values: None
//
/*********************************************************************/
// Standard Deviation Calculation
void   StandardDev(double Voltage, double *stDv, double *rmsBits,
				   double *sdTot, double *sd, double *Average, int *sdCount)
{		//standard deviation and RMS resolution calculation
	int nnn;						// loop counter 
	int sdGrp=20;    	     	// Number of voltage samples to include in group
	double XXX;
	sd[*sdCount] = Voltage;	 //  Place latest voltage into array
	(*sdTot) += Voltage;		 //  Add latest voltage to total
	if ((*sdCount) == sdGrp) //  Check if all readings for group have been received 
	{
		(*stDv) = 0;				
		(*sdTot) = (*sdTot) / sdGrp;		
		for (nnn = 1; nnn != sdGrp; nnn++)	
		{	
			(*stDv) = (*stDv) + ((*sdTot) - sd[nnn])*((*sdTot) - sd[nnn]);
		}		
		(*stDv) = sqrt((*stDv)/(sdGrp-1));  
		XXX =((double)((*stDv)/M203.gain));    // calculate RMS noise 
		if  ( XXX < 1) XXX = 1;
		(*rmsBits) = M203.totalBits - ((logl(XXX)) * 1.442695); 
		(*Average) = (((double)(*sdTot)));
		(*stDv) = ((double)(*stDv));
		(*sdTot)=0; (*sdCount) = 1;
	}
	else	
	{
		(*sdCount)++;
	}
	return;
}

//********************************************************************/
// SCAN MENU - This function displays the scanning menu and processes
//	 scanning parameter input including scan interval adjustment, data 
//  log file selection, and scanning channel selection.  
//
// Input: M203 structure variables
// Output - Global Variables: ScanMode, M203.scanInterval, M203.scanChan
// Output - Return Values:
//   0 = Start Scanning with current selections 
//   1 = Do not start Scanning and return to polled mode 
//
/*********************************************************************/
short ScanMenu()
{
	int channel;
	double NewScanInterval; 
	double MaxTime, MinTime;
	short temp;
	int ScanKeyInput;
	char tmpbuf4[80];
	static char ScanTypeBuf[80]= "          Normal           ";

	clrscr();

SCANMENU1:
	ClearCommandLines();
	gotoxy(22,2); printf ("MODEL 203 Data Acquisition System");
	gotoxy(27,4); printf("SCANNING CONFIGURATION");
	gotoxy(18,7); printf("C) select scan Channel             %d",M203.scanChan);
	gotoxy(18,9); printf("I) select scan Interval  %11.4lf seconds",M203.scanInterval);
	gotoxy(18,11); printf("T) select scan Type   ");
	puts(ScanTypeBuf);
	gotoxy(18,13); printf("F) log File                     ");
	printf("                     ");
	gotoxy(51,13);
	puts(FileName);
	gotoxy(14,15); printf("Enter) perform the scan");
	gotoxy(16,17); printf("Esc) return to polled mode");
	ScanKeyInput = getch();
	if (islower(ScanKeyInput)) ScanKeyInput = toupper(ScanKeyInput);
	switch (ScanKeyInput)
	{
		case 'C':						// Select Scanning channel 
			channel=-1;
			while ((channel !='0')&&(channel !='1')&&(channel!='6')&&(channel !='7'))
			{
				gotoxy(1,24);  printf("  enter channel 0,1,6, or 7 : ");
				channel = getch();
				if ((channel == 27)||(channel==13)) goto SCANMENU1;
			}
			M203.scanChan = ((short)(channel - '0'));
			break;

		case 'I':					// Select Scan Interval
			NewScanInterval = -1;
			temp = CheckScanTime(&NewScanInterval,&MaxTime,&MinTime);
			while (temp !=0)
			{
				gotoxy(1,25);  
				printf(" minimum: %10.4lf seconds   maximum: %12.4lf seconds : ",MinTime,MaxTime);
				gotoxy(1,24);  
				printf(" Enter new Scan Interval between limits : ");
				SetWin32CursorOn();
				getch();            
				fgets(tmpbuf4,80,stdin);
				if(tmpbuf4[0]=='\n')
				{
					ClearCommandLines();
					goto SCANMENU1;		// No input, No Change
				}
				sscanf(tmpbuf4,"%lf",&NewScanInterval);
				temp = CheckScanTime(&NewScanInterval,&MaxTime,&MinTime);
				SetWin32CursorOff();
				if (NewScanInterval == 27) goto SCANMENU1;
				if (temp != 0)
				{
					gotoxy(1,24);  
					printf("scan time out of limits  press any key to continue");
					getch();            
					ClearCommandLines();
				}
			}
			break;

		case 'T':      				// Select Scanning Mode
			if(++ScanMode==6) ScanMode = 0;
			switch (ScanMode)
			{
				case 0: 
					sprintf(ScanTypeBuf,"          normal           ");
					ScanToken=0x8B; Edge=0;
					break;
		
				case 1: 
					sprintf(ScanTypeBuf,"         checkSum          "); 
					ScanToken=0x8D; Edge=0;
					break;

				case 2: 
					sprintf(ScanTypeBuf," positive_triggered_normal "); 
					ScanToken=0x89; Edge=0;
					break;
		
				case 3: 
					sprintf(ScanTypeBuf," negative_triggered_normal "); 
					ScanToken=0x89; Edge=1;
					break;
		
				case 4: 
					sprintf(ScanTypeBuf,"positive_triggered_checksum"); 
					ScanToken=0x8C; Edge=0;
					break;
		
				case 5: 
					sprintf(ScanTypeBuf,"negative_triggered_checksum"); 
					ScanToken=0x8C; Edge=1;
					break;
			}//End of Switch
			break;

		case 'F':            		// Input Data Logging File Name
			gotoxy(1,24);	printf("  enter name for log file name (enter for none) : ");
			getch();
			gets(NewFileName);
			if(NewFileName[0]=='\n')
			{
				ClearCommandLines();
				goto SCANMENU1;		// No input, No Change
			}
			if (strcmp(NewFileName,FileName)!=0) // if the same, do nothing
			{
				if (strlen(FileName)!=0)			// Close Old file if open
				{				
					if (LogFile!=0)
					if (fclose(LogFile)==0)
					{
						LogFile = 0;
						fLogToFile=0;
					}
					else
					{
						printf("file close error");
						getch();
					}
				}
				if (strlen(NewFileName)!=0)		// if Not a Null String
				{									
					if ((LogFile = fopen(NewFileName, "wt"))== NULL)
					{
						fLogToFile=0;
						gotoxy(1,24);
						printf("could not open/create Log File");
						getch();
						goto SCANMENU1;
					}
					else
					{
						sprintf(FileName,"           ");
						strncpy(FileName,NewFileName,12);   
						fLogToFile = 1;
					}
				}
				else
				{
					sprintf(FileName,"<none>   ");
					fLogToFile=0;
				}
			}
			break;

		case 13:				// <Enter> Start Scanning with selected parameters
			if(fLogToFile==1)
			{  						// Add header to log file
				fprintf(LogFile,"file: %-12s  channel: %-1d\n",FileName,M203.scanChan);
				fprintf(LogFile,"scan interval: %-9.3lf  scan type: %-s\n",M203.scanInterval,ScanTypeBuf);
				fprintf(LogFile,"  SCAN     COUNT    MILLIVOLTS\n");
			}
			temp = CheckScanTime(&M203.scanInterval,&MaxTime,&MinTime);
			if (temp != 0)
			{
				gotoxy(3,24);  
				printf("scan time out of limits  press any key to continue");
				getch();            
				ClearCommandLines();
				break;
			}
			else
			{
				return 0;
			}

		case 27:			// <Esc> Do Not Start Scanning - Return to Polled Mode
			M203.dataAcqMode=1;		// Reselect Polled Mode
			return 1;

		default:
			goto SCANMENU1;
	} // end of switch
	goto SCANMENU1;
}

//********************************************************************/
//  SELECT DATA RATE - This functions sets the data sampling rate used
//		in polled mode.
//
// Input: None
// Output - Global Variables: None
// Output - Return Values: None
//
/*********************************************************************/
void SelectRate()    //select the sampling rate for the A/D
{
	float newRate=-1;
	char tmpbuf5[80];
	SetWin32CursorOn();
	ClearCommandLines();
	gotoxy(3,24);printf("enter sample rate between %2.1f and %4.1f Hz : ",
			(M203.maxRate/255),M203.maxRate);
	getch();             
	fgets(tmpbuf5,80,stdin);
	if(tmpbuf5[0]=='\n') return;		// No input, No Change
	sscanf(tmpbuf5,"%f",&newRate);
	SetWin32CursorOff();
	if (newRate > M203.maxRate)
	{
		newRate = ((float)(M203.maxRate));
	}
	else if (newRate < (M203.maxRate/255))
	{
		newRate = (M203.maxRate/255);
	}
	M203.totalPeriods = floor(((1/newRate)*M203.maxRate)+ 0.5);
	if (M203.totalPeriods == 0) M203.totalPeriods = 1;
	gotoxy (28,6); printf ("%4.1f",M203.maxRate/M203.totalPeriods);
	ClearCommandLines();
	gotoxy(3,24);printf(" performing System Calibration ...                                    ");
	SendADMode();
	SysCal();
	return;
}

//********************************************************************/
// READ CONFIGURATION FILE - This function reads the configuration file,
//		if it is in the current directory and initializes Model 203
//		variables.  If the configuration file is not found, default 
//		configuration values are used.	
//
// Input: 
// Output - Global Variables: 
// Output - Return Values:
//   0 = Configuration file values were used for initialization
//   1 = Default values were used for initialization 
//
/*********************************************************************/
short ReadConfig()  
{
	FILE *rCfg;
	if ((rCfg = fopen("M203.CFG", "rt")) != NULL)        
	{
		// (6 parameters) Configuration file Line 1 
		fgets(InputString,80,rCfg);
		StringSeg(1,InputString,OutputString);		
		sscanf(OutputString,"%hd",&M203.modeRegHi);
		short val = M203.modeRegHi;
		StringSeg(2,InputString,OutputString);		
		sscanf(OutputString,"%hd",&M203.modeRegMid);   
		StringSeg(3,InputString,OutputString);		
		sscanf(OutputString,"%hd",&M203.modeRegLo);  
		StringSeg(4,InputString,OutputString);		
		sscanf(OutputString,"%hd",&M203.pollChan);      
		StringSeg(5,InputString,OutputString);		
		sscanf(OutputString,"%hd",&M203.subSample);      
		StringSeg(6,InputString,OutputString);		
		sscanf(OutputString,"%hd",&M203.divideBy);
		
		// (9 parameters) Configuration file Line 2 
		fgets(InputString,80,rCfg);                   
		StringSeg(1,InputString,OutputString);		
		sscanf(OutputString,"%hd",&M203.dacEnable);      
		StringSeg(2,InputString,OutputString);		
		sscanf(OutputString,"%hd",&M203.baud);         
		StringSeg(3,InputString,OutputString);		
		M203.comPort=(short)(OutputString[4]-'1');
		StringSeg(4,InputString,OutputString);		
		sscanf(OutputString,"%hu",&M203.inputBuffer);      
		if (M203.inputBuffer<11) M203.inputBuffer=11;
		if (M203.inputBuffer>32767) M203.inputBuffer=32767;
		StringSeg(5,InputString,OutputString);		
		sscanf(OutputString,"%hd",&spare);      
		StringSeg(6,InputString,OutputString);		
		sscanf(OutputString,"%hd",&M203.bU);    
		StringSeg(7,InputString,OutputString);		
		sscanf(OutputString,"%f",&M203.gain1);         
		StringSeg(8,InputString,OutputString);		
		sscanf(OutputString,"%f",&M203.maxRate);         
		StringSeg(9,InputString,OutputString);		
		sscanf(OutputString,"%hd",&M203.wL);
		
		// (4 parameters) Configuration file Line 3 
		fgets(InputString,80,rCfg);                   
		StringSeg(1,InputString,OutputString);		
		sscanf(OutputString,"%hd",&placeholder);      
		StringSeg(2,InputString,OutputString);		
		sscanf(OutputString,"%hd",&placeholder);      
		StringSeg(3,InputString,OutputString);		
		sscanf(OutputString,"%hd",&placeholder);      
		StringSeg(4,InputString,OutputString);		
		sscanf(OutputString,"%hd",&placeholder);
		
		// (3 parameters) Configuration file Line 4 
		fgets(InputString,80,rCfg);                   
		StringSeg(1,InputString,OutputString);		
		sscanf(OutputString,"%hd",&placeholder);      
		StringSeg(2,InputString,OutputString);		
		sscanf(OutputString,"%hd",&placeholder);      
		StringSeg(3,InputString,OutputString);		
		sscanf(OutputString,"%hd",&M203.scanChan);
		
		// (6 parameters) Configuration file Line 5 
		fgets(InputString,80,rCfg);                   
		StringSeg(1,InputString,OutputString);		
		sscanf(OutputString,"%uc",&M203.scan);   
		StringSeg(2,InputString,OutputString);		
		sscanf(OutputString,"%uc",&M203.scan1);   
		StringSeg(3,InputString,OutputString);		
		sscanf(OutputString,"%uc",&M203.scan2);   
		StringSeg(4,InputString,OutputString);		
		sscanf(OutputString,"%lf",&M203.scanInterval);
		StringSeg(5,InputString,OutputString);		
		sscanf(OutputString,"%hd",&M203.tScan);        
		StringSeg(6,InputString,OutputString);		
		sscanf(OutputString,"%hd",&reserved);
		
		// (8 parameters) Configuration file Line 6 
		fgets(InputString,80,rCfg);                   
		StringSeg(1,InputString,OutputString);		
		sscanf(OutputString,"%f",&M203.dac1volts);       
		StringSeg(2,InputString,OutputString);		
		sscanf(OutputString,"%f",&M203.dac2volts);       
		StringSeg(3,InputString,OutputString);		
		sscanf(OutputString,"%hd",&M203.totalPeriods);      
		StringSeg(4,InputString,OutputString);		
		if((OutputString=="\"TEXT\"")||(OutputString=="\"text\""))
		{
			scandisp=0;      
		}
		else
		{
			scandisp=1;  // "graphics"    
		}		
		StringSeg(5,InputString,OutputString);		
		sscanf(OutputString,"%hd",&posfs);      
		StringSeg(6,InputString,OutputString);		
		sscanf(OutputString,"%hd",&negfs);      
		StringSeg(7,InputString,OutputString);		
		sscanf(OutputString,"%hd",&vdiv);      
		StringSeg(8,InputString,OutputString);		
		sscanf(OutputString,"%hd",&hdiv);      
		// (3 parameters) Configuration file Line 7 
		fgets(InputString,80,rCfg);                   
		StringSeg(1,InputString,OutputString);		
		sscanf(OutputString,"%f",&M203.gain);  
		StringSeg(2,InputString,OutputString);		
		sscanf(OutputString,"%ld",&M203.runningAvg);            
		StringSeg(3,InputString,OutputString);		
		sscanf(OutputString,"%hd",&M203.timeBase);            
		fclose(rCfg);
		if (M203.wL==128)
		{
			M203.wordCnt=3;		// 3 for 24-bit)
		}
		else
		{
			M203.wordCnt=2;
		}
		return 0;
	}
	else
	{
		scandisp=0;      		// Text Scan display mode
		reserved=0;       		// 
		placeholder=0;			// 
		M203.comPort=0;			// COM1
		M203.scanInterval=1;	// 1 sec
		M203.numChannels=1;		// 1 channel
		M203.tScan=0;  			// 0 Single Channel Scanning
		M203.scanChan=7;		// 0 Input Channel - Scanning
		M203.lastDigInp=255;  	// All High Last digital input value
		M203.version=203;	  	// Model 203 version (203)
		M203.pollChan=7;		// 7 Offset Channel
		M203.subSample=160;   	// 60 Hz Value, 192 for 50 Hz
		M203.rawCount=0;      	// 0 Last raw count
		M203.digOutWord=256;	// 256 Non-Valid Value - Last value sent (0-255)
		M203.wordCnt=3;      	// 3 for 24-bit)
		M203.errorType=0;    	// error code returned from DLL
		M203.bU=1;           	// UniPolar = 0, BiPolar=1 (M203 always BiPolar)
		M203.filter=0;    	  	// Not used      
		M203.divideBy=0;     	// 0 Remote (power of 2) division (/1)	       
		M203.gain1=1;         	// 
		M203.gain=1;         	// 
		M203.maxRate=120;		// Maximum sample rate (120 Hz for 60Hz)
		M203.modeRegHi=0;    	// Not used
		M203.modeRegMid=128;	// MSB = 1, (24 bit) word length 
		M203.modeRegLo=2; 		//  
		M203.totalPeriods=2; 	// 2 for 60 HZ Sample Rate (with 120 maxRate) 
		M203.baud=0;         	// 0=9600 bps 
		M203.dac1volts=0;    	// Current Voltage for Analog Output 1
		M203.dac2volts=0;    	// Current Voltage for Analog Output 2
		M203.dacEnable=0;    	// Not Used 
		M203.timeBase=96;  	  	// 256 - SubSample 
		M203.echoTestCount=0; 	// Serial Echo Test Byte Count
		M203.runningAvg=1;		//
		M203.dataAcqMode=1;		// 1 = Polled
		M203.wL=128;
		M203.inputBuffer=32767;
		posfs=0;      
		negfs=0;      
		vdiv=0;      
		hdiv=0;      
		return 1;
	}
}

//********************************************************************/
// WRITE CONFIGURATION FILE - This functions saves the current Model 203
//		setup in the configuration file, M203.CFG, in current directory.
//
// Input: M203 variables
// Output - Global Variables: 
// Output - Return Values:
//   0 = No Errors - Configuration file saved successful
//   1 = Could not open a configuration file
//
/*********************************************************************/
short WriteConfig()
{
	FILE *wCfg;
	if ((wCfg = fopen("M203.CFG", "wt")) != NULL)        
	{
		//1st line
//  WRITE #2, MODEREGHI%, MODEREGMID%, MODEREGLO%, Channel%, SUBSAMPLE%, SPARE%
		fprintf(wCfg,"%d,",M203.modeRegHi);    
		fprintf(wCfg,"%d,",M203.modeRegMid);   
		fprintf(wCfg,"%d,",M203.totalPeriods);
		fprintf(wCfg,"%d,",M203.pollChan);      
		fprintf(wCfg,"%d,",M203.subSample);      
		fprintf(wCfg,"%d\n",M203.divideBy);      

		//2nd line
//  WRITE #2, DACENABLE%, BAUD%, COM$, INPbUF%, SPARE%, bU%, GAIN%, maxRate, wL%
		fprintf(wCfg,"%d,",M203.dacEnable);      
		fprintf(wCfg,"%d,",M203.baud);         
		fprintf(wCfg,"\"COM%d\":,",M203.comPort+1);      
		fprintf(wCfg,"%d,",M203.inputBuffer);      
		fprintf(wCfg,"%d,",spare);      
		fprintf(wCfg,"%d,",M203.bU);      
		fprintf(wCfg,"%f,",M203.gain1);         
		fprintf(wCfg,"%f,",M203.maxRate);         
		fprintf(wCfg,"%d\n",M203.wL);
		
		//3rd line
//  WRITE #2, PLACEHOLDER%, PLACEHOLDER%, PLACEHOLDER%, PLACEHOLDER%
		fprintf(wCfg,"%d,",placeholder);      
		fprintf(wCfg,"%d,",placeholder);      
		fprintf(wCfg,"%d,",placeholder);      
		fprintf(wCfg,"%d\n",placeholder);      

		//4th line
//  WRITE #2, PLACEHOLDER%, PLACEHOLDER%, scanChan%
		fprintf(wCfg,"%d,",placeholder);      
		fprintf(wCfg,"%d,",placeholder);      
		fprintf(wCfg,"%d\n",M203.scanChan);
		
		//5th line
//  WRITE #2, SCAN%, SCAN1%, SCAN2%, scanInterval#, STYPE%, RESERVE%
		fprintf(wCfg,"%d,",M203.scan);   
		fprintf(wCfg,"%d,",M203.scan1);   
		fprintf(wCfg,"%d,",M203.scan2);   
		fprintf(wCfg,"%lf,",M203.scanInterval);
		fprintf(wCfg,"%d,",M203.tScan);        
		fprintf(wCfg,"%d\n",reserved);      
//  WRITE #2, Dac1Volts, Dac2Volts, totalPeriods%, ScanDisp$, POSFS, NEGFS, VDIV, HDIV
		fprintf(wCfg,"%f,",(M203.dac1volts));       
		fprintf(wCfg,"%f,",(M203.dac2volts));       
		fprintf(wCfg,"%d,",M203.totalPeriods);      
		if (scandisp==0)
		{
			fprintf(wCfg,"\"Text\",");      
		}
		else
		{
			fprintf(wCfg,"\"Graphics\",");      
		}
		fprintf(wCfg,"%d,",posfs);      
		fprintf(wCfg,"%d,",negfs);      
		fprintf(wCfg,"%d,",vdiv);      
		fprintf(wCfg,"%d\n",hdiv);      
//  WRITE #2, GAIN#, runningAvg%, TimeBase
		fprintf(wCfg,"%f,",M203.gain);  
		fprintf(wCfg,"%d,",M203.runningAvg);        
		fprintf(wCfg,"%d\n",M203.timeBase);  
		fclose(wCfg);
		fclose(wCfg);
		return 0;
	}
	return 1;
}


//********************************************************************/
//
//turns on cursor in Win32 console mode
//
// Input: None
// Output - Global Variables: None
// Output - Return Values: None
/*********************************************************************/
void SetWin32CursorOn(void)
{
	ConsoleCursorInfo.bVisible = TRUE;
	SetConsoleCursorInfo(hOutCon, &ConsoleCursorInfo);
}

//********************************************************************/
//
//turns off cursor in Win32 console mode
//
// Input: None
// Output - Global Variables: None
// Output - Return Values: None
/*********************************************************************/
void SetWin32CursorOff(void)
{
	ConsoleCursorInfo.bVisible = FALSE;
	SetConsoleCursorInfo(hOutCon, &ConsoleCursorInfo);
}


//********************************************************************/
// PRINT TOP MENU SCREEN
//
// Input: None
//
// Output - Global Variables: None
// Output - Return Values:
//   0 = No Errors -
//   1 = 
//
/*********************************************************************/
void PrintTopMenuScreen()
{
	clrscr();
	gotoxy(22,2); printf ("MODEL 203 Data Acquisition System");
	gotoxy(24,4); printf ("SERIAL COMMUNICATIONS MENU");
	gotoxy(20,7); printf ("1) BAUD rate               %4d",bauds[M203.baud]);
	gotoxy(20,9); printf ("2) Select port             COM%d",M203.comPort+1);
	gotoxy(20,11);	printf ("3) Input buffer           %6d bytes",M203.inputBuffer);
	gotoxy(20,13);	printf ("4) Serial check");
	gotoxy(20,15);	
	if(fDebug == 0)
		printf ("5) Create DLL Debug file   none");
	if(fDebug == 1)
		printf ("5) Create DLL Debug file   low");
	if(fDebug == 2)
		printf ("5) Create DLL Debug file   high");
	gotoxy(16,17);	printf ("Enter) Start");
	gotoxy(18,19);	printf ("Esc) Exit program");
	return;
}


//********************************************************************/
// Clear Command Lines - Clears last to lines of screen
//
// Input: None
//
// Output - Global Variables: None
// Output - Return Values:
//   0 = No Errors -
//   1 = 
//
/*********************************************************************/
void ClearCommandLines(void)
{
	gotoxy (1,23); printf ("                                                                      ");
	gotoxy (1,24); printf ("                                                                      ");
	gotoxy (1,25); printf ("                                                                      ");
	return;
}

//********************************************************************/
// PRINT Model 202 ACQUISITION SCREEN
//
// Input: None
//
// Output - Global Variables: None
// Output - Return Values:
//   0 = No Errors -
//   1 = 
//
/*********************************************************************/
void PrintPolledScreen()	
{
	clrscr();
	gotoxy (3,1); 
	printf ("POLLED  MENU          Lawson Labs, Inc.  Model 203");
	gotoxy (60,1);
	printf ("Version = %3d\n",M203.version);

	gotoxy (5,4); printf ("C) A/D input Channel     %d", M203.pollChan);
	gotoxy (5,5); printf ("L) Local averaging     %3d       ", M203.runningAvg);
	gotoxy (5,6); printf ("R) sample Rate         %5.1lf",M203.maxRate/M203.totalPeriods);
	gotoxy (5,7); printf ("S) System calibration");
	// Right Column of Screen (x = 43-to-80, y = 2-to-22)
	gotoxy (40,4); printf ("D) Digital inputs ");
	gotoxy (40,5); printf ("U) set digital oUtput             ");
	gotoxy (68,5); 
	if((M203.digOutWord >=0)&&(M203.digOutWord<=255)) printf("%3d",M203.digOutWord);
	gotoxy (40,6); printf ("K) get checKsum   	  "); 
	gotoxy (40,7); printf ("O) Offset calibration");
	gotoxy (40,8); printf ("F) Full-scale calibration");
	gotoxy (40,10); printf ("1) analog output1 : %7.3f mv",(M203.dac1volts*1000));
	gotoxy (40,11); printf ("2) analog output2 : %7.3f mv",(M203.dac2volts*1000));

	gotoxy (7,12); printf ("COUNT      MILLIVOLTS");
	gotoxy (3,24); printf ("SPACE) reset max/min/delta,  N) scaN,  ESC) serial menu");
	gotoxy (40,22); 
}

//********************************************************************/
// Do Command
//
// Input: None
//
// Output - Global Variables: None
// Output - Return Values:
//   0 = No Errors -
//   1 = 
//
//*********************************************************************/
void  DoCommand(int keyinput)
{
	float	NewVoltage;
	short	temp;
	long	OldAverage;
	long	temp1;
	int		tempCh;
	char	tmpbuf2[80];

	switch(keyinput)
	{
		case ('C'):
			tempCh = -1;
			while ((tempCh != '0')&& (tempCh != '1')&&(tempCh != '6')&&(tempCh!='7'))
			{
				ClearCommandLines();
				gotoxy(1,24);  printf("  enter channel (0,1,6, or 7) : ");
				tempCh = getch();
				if(tempCh==13)
				{
					ClearCommandLines();
					goto EndDoCommand;		// No input, No Change
				}
			}
			M203.pollChan = ((short)(tempCh - '0'));
			SetChanCode();
			gotoxy(30,4);printf("%1d",M203.pollChan);
			ClearVoltageArray();
			ClearSD();
			ClearCommandLines();
			break;

		case ('D'):
			GetDigIn(0x80);
			gotoxy (67,4);
			if(((M203.lastDigInp)&0x0001)==1)
			{
				printf ("Inactive");
			}
			else
			{
				printf (" Active   ");
			}
			break;

		case ('K'):						// Get running checksum from M203
			gotoxy (63,6);				// printf ("K) checKsum         ");
			if (GetCheckSum() == 0)
			{
				printf ("     OK");
			}
			else
			{
				printf (" not OK");
			}
			break;

		case ('1'):
			NewVoltage=-1;
			while (NewVoltage < 0 || NewVoltage > 5000)
			{
				SetWin32CursorOn();
				ClearCommandLines();
				gotoxy(1,24);
				printf("  enter analog output 1 voltage (0 - 5000 mv) : ");
				getch();
				fgets(tmpbuf2,80,stdin);
				SetWin32CursorOff();
				if(tmpbuf2[0]=='\n')
				{
					ClearCommandLines();
					goto EndDoCommand;		// No input, No Change
				}
				sscanf(tmpbuf2,"%f",&NewVoltage);
			}
			NewVoltage = NewVoltage/1000;	//calculate new voltage in volts
			M203.dac1volts = NewVoltage;
			ChangeAnalogOutput(DAC_1);      //set the voltage
			gotoxy (40,10); 
			printf ("1) analog output1 :  %7.3f mv   ",(M203.dac1volts*1000));
			ClearCommandLines();
			break;

		case ('2'):
			NewVoltage=-1;
			while (NewVoltage < 0 || NewVoltage > 5000)
			{
				SetWin32CursorOn();
				ClearCommandLines();
				gotoxy(1,24);
				printf("  enter analog output 2 voltage (0 - 5000 mv) : ");
				getch();
				fgets(tmpbuf2,80,stdin);
				SetWin32CursorOff();
				if(tmpbuf2[0]=='\n')
				{
					ClearCommandLines();
					goto EndDoCommand;		// No input, No Change
				}
				sscanf(tmpbuf2,"%f",&NewVoltage);
			}
			NewVoltage = NewVoltage/1000;	//calculate new voltage in volts
			M203.dac2volts = NewVoltage;
			ChangeAnalogOutput(DAC_2);      //set the voltage
			gotoxy (40,11); 
			printf ("2) analog output2 :  %7.3f mv   ",(M203.dac2volts*1000));
			ClearCommandLines();
			break;

		case ('R'):
			SelectRate();
			ClearVoltageArray();
			ClearSD();
			ClearCommandLines();
			break;


		case ('L'):							// Set Local Averaging
			OldAverage = M203.runningAvg;
			temp1 = -1;
			while ((temp1 < 1)||(temp1 > 32767))
			{
				SetWin32CursorOn();
				ClearCommandLines();
				gotoxy(3,24); printf("  enter local averaging : ");
				getch();
				fgets(tmpbuf2,80,stdin);
				SetWin32CursorOff();
				if(tmpbuf2[0]=='\n')
				{
					ClearCommandLines();
					goto EndDoCommand;		// No input, No Change
				}
				sscanf(tmpbuf2,"%ld",&temp1);
			}
			M203.runningAvg=temp1;	
			gotoxy (25,5); printf ("   %3d    ", M203.runningAvg);
			if (OldAverage != M203.runningAvg)
			{
				ClearVoltageArray();
				ClearCommandLines();
				gotoxy(1,24); 	printf ("  performing system calibration ...");
				SysCal();
			}
			ClearVoltageArray();
			ClearSD();
			ClearCommandLines();
			break;
		
		case ('S'):			// Perform M203 System Calibration
			ClearCommandLines();
			gotoxy(1,24); 	printf ("  performing system calibration ...");
			SysCal();
			ClearVoltageArray();
			ClearSD();
			ZeroDelta();
			ClearCommandLines();
			break;

		case ('O'):			// Perform Offset Calibration
			ClearCommandLines();
			gotoxy(1,24); 	printf ("  performing offset calibration ...");
			ClearVoltageArray();
			OffsetCal();
			ClearVoltageArray();
			ClearSD();
			ClearCommandLines();
			break;

		case ('F'):
			ClearCommandLines();
			gotoxy(1,24); 	printf ("  performing full-scale calibration ...");
			temp=FullScaleCal();
			if ((temp==4)||(temp==1))
			{
				ClearCommandLines();
				gotoxy(1,24); 	
				printf (" Too small a range for calibration. Press any key to continue");
				getch();	
			}
			else
			{
				ClearVoltageArray();
				ClearSD();
				ClearCommandLines();
			}
			break;	

		case (' '): 					// <space> Reset Max-Min display
			ZeroDelta();
			break;

		case ('U'):						// Set Digital Output
			temp=256;
			while ((temp < 0) || (temp > 255))
			{
				SetWin32CursorOn();
				ClearCommandLines();
				gotoxy(3,24); 	printf ("Enter output code (0 - 255): ");
				getch();
				fgets(tmpbuf2,80,stdin);
				SetWin32CursorOff();
				if(tmpbuf2[0]=='\n')
				{
					ClearCommandLines();
					goto EndDoCommand;	// No input, No Change
				}
				sscanf(tmpbuf2,"%d",&temp);
			}
			M203.digOutWord = temp;
			SetDigOut();
			gotoxy (68,5); printf ("%3d",M203.digOutWord);
			ClearCommandLines();
			break;

		default:
			break;
	} // end of switch
EndDoCommand:
	gotoxy (3,24); printf ("SPACE) reset max/min/delta,  N) scaN,  ESC) serial menu");
	return;
} // end of function

//********************************************************************/
// Get Voltage 
//
// Input: None
//
// Output - Global Variables: None
// Output - Return Values:
//   0 = No Errors -
//   1 = 
//
/*********************************************************************/
short GetVoltage_LOCAL(double *Voltage)
{                 
	if(GetConversion()) return 1;
	*Voltage = M203.Voltage;

	return 0;
}


//********************************************************************/
// Clear SD - Clears Standard Deviation Variables 
//
// Input: None
// Output - Global Variables: stDv, rmsBits, sdTot, Average, sd, sdCount
// Output - Return Values: none
//
/*********************************************************************/
void ClearSD(void)
{
	stDv=0; rmsBits=0; sdTot = 0; Average =0; sd[41]; sdCount=1;
	return;
}

//********************************************************************/
// Reset Max and Min - Set Max to negative range limit and Min
//  to positive range limit. 
//
// Input: None
// Output - Global Variables: Max, Min
// Output - Return Values: None
//
/*********************************************************************/
void ZeroDelta(void)
{
	Max=-10000;
	Min=10000;
	return;
}

//********************************************************************/
// Print Error - This function checks and prints an error message for 
//		any of the 6 Model 203 errors.
//
//  Error
//	 Type
//
//   1 	"ChkSum error"
//   2 	"CmdPending error"
//   5 	"sign-on error"
//   6 	"BAUD error"         
//   8 	"Pass through command error"
//   9 	"Data request error"
// 
//	Input: None
// Output - Global Variables: Max, Min
// Output - Return Values: 
//   0 = no Model 203 errors
//	  1 = a Model 203 error has been detected and indicated	
/*********************************************************************/
short PrintError()
{
	int keyinput=25;
	clrscr(); 
	switch(M203.errorType)
	{
		case 0:
			return 0;

		case 1:
			gotoxy(3,2); printf ("ERROR: Model 203 detected a checksum error");
			break;

		case 2:
			gotoxy(3,2); printf ("ERROR: Model 203 command pending error");
			break;

		case 5:
			gotoxy(3,2); printf ("ERROR: sign on error");
			break;

		case 6:
			gotoxy(3,2); printf ("ERROR: BAUD rate error");
			break;

		case 8:
			gotoxy(3,2); printf ("ERROR: pass-through command error");
			break;

		case 9:
			gotoxy(3,2); printf ("ERROR: data request error");
			break;
		default:
			break;
	}
	gotoxy(3,25); 
	printf ("press Q to restart. Your last-saved .CFG file will be read from the disk.");
	while (keyinput!='Q')
	{
		keyinput=getch();
		if (islower(keyinput)) keyinput = toupper(keyinput);
	}
	return 1;
} 

//********************************************************************/
// StringSeg - This utility routine segments a string from a given input
//  string returning the nth parameter in the input string delimited by 
//	 commas. 
//
// Input String   Parameter 1 , Prameter 2 , ..... Prameter N \n
//
// 
//	Input: 
//			Parameter to return starting with 1
//			Pointer to Input String
//			Pointer to Output String
//
// Output - Global Variables: Max, Min
// Output - Return Values: None
//
/*********************************************************************/
void StringSeg(int PIndex, char* ISptr, char* OSptr)
{
	char *CommaPlace;
	int  i;
	for (i=0;i<PIndex;i++)
	{
		if (i==PIndex-1)		// Check for Last Parameter
		{
			strcpy(OSptr,ISptr);
		}
		else
		{
			CommaPlace = strchr(ISptr,',');
			strncpy(OSptr,ISptr,((size_t)(CommaPlace-ISptr)));
			ISptr=CommaPlace+1;
		}
	}
	return;	
}

//********************************************************************/
// Wait - Utility Delay routine 
//
//  CLK_TCK is a system variable in clock()ticks/sec 
//
// Input: DelayInterval = desired delay time in milliseconds
// Other Routines Called: None
// Output - Global Variables: 
// Output - Return Values:
//   0 = No Errors
//   1 = 
//
/*********************************************************************/
void Wait(float milliseconds)
{
	clock_t end;
	end = (clock() + ((clock_t)((milliseconds*CLK_TCK)/1000)));
	while(clock() < end);
	return;
}

//==== LOADLIBRARIES FUNCTION =================================================
BOOLEAN
LoadLibraries()
{
    hLibM203 = LoadLibrary ("NT_m203.dll");
    if(!hLibM203)
	{
		MessageBox(0, "Error loading NT_m203.DLL", "-- error", MB_OK);
		return FALSE;
	}
	return TRUE;
}

//==== UNLOADLIBRARIES FUNCTION ===============================================
BOOLEAN
UnloadLibraries()
{
	FreeLibrary(hLibM203);
    hLibM203 = NULL;
	return TRUE;
}

//==== CHECKLIBFUNCS FUNCTION =================================================
BOOLEAN
CheckLibFuncs()
{
	BOOLEAN Status = TRUE;

    SetPtrs = (DLL_SET_PTRS)GetProcAddress(hLibM203, "SetPtrs");
    if(!SetPtrs)
    {
        MessageBox(0, "failed to find function:\n     SetPtrs(...)"
                "\nin M201_98.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		FreeLibrary(hLibM203);
        return FALSE;
    }
    SignOn = (DLL_SIGN_ON)GetProcAddress(hLibM203, "SignOn");
    if(!SignOn)
    {
        MessageBox(0, "failed to find function:\n     SignOn()"
                "\nin M201_98.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		FreeLibrary(hLibM203);
        return FALSE;
    }
    EchoTest = (DLL_ECHO_TEST)GetProcAddress(hLibM203, "EchoTest");
    if(!EchoTest)
    {
        MessageBox(0, "failed to find function:\n     EchoTest()"
                "\nin M201_98.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		FreeLibrary(hLibM203);
        return FALSE;
    }

    GetDigIn = (DLL_GET_DIG_IN)GetProcAddress(hLibM203, "GetDigIn");
    if(!GetDigIn)
    {
        MessageBox(0, "failed to find function:\n     GetDigIn(...)"
                "\nin M201_98.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		FreeLibrary(hLibM203);
        return FALSE;
    }
    GetVer = (DLL_GET_VER)GetProcAddress(hLibM203, "GetVer");
    if(!GetVer)
    {
        MessageBox(0, "failed to find function:\n     GetVer(...)"
                "\nin M201_98.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		FreeLibrary(hLibM203);
        return FALSE;
    }
    SetDigOut = (DLL_SET_DIG_OUT)GetProcAddress(hLibM203, "SetDigOut");
    if(!SetDigOut)
    {
        MessageBox(0, "failed to find function:\n     SetDigOut(...)"
                "\nin M201_98.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		FreeLibrary(hLibM203);
        return FALSE;
    }
    FullScaleCal = (DLL_FULL_SCALE_CAL)GetProcAddress(hLibM203, "FullScaleCal");
    if(!FullScaleCal)
    {
        MessageBox(0, "failed to find function:\n     FullScaleCal(...)"
                "\nin M201_98.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		FreeLibrary(hLibM203);
        return FALSE;
    }
    OffsetCal = (DLL_OFFSET_CAL)GetProcAddress(hLibM203, "OffsetCal");
    if(!OffsetCal)
    {
        MessageBox(0, "failed to find function:\n     OffsetCal(...)"
                "\nin M201_98.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		FreeLibrary(hLibM203);
        return FALSE;
    }
    SysCal = (DLL_SYS_CAL)GetProcAddress(hLibM203, "SysCal");
    if(!SysCal)
    {
        MessageBox(0, "failed to find function:\n     SysCal(...)"
                "\nin M201_98.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		FreeLibrary(hLibM203);
        return FALSE;
    }
    GetConversion = (DLL_GET_CONVERSION)GetProcAddress(hLibM203, "GetConversion");
    if(!GetConversion)
    {
        MessageBox(0, "failed to find function:\n     GetConversion(...)"
                "\nin M201_98.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		FreeLibrary(hLibM203);
        return FALSE;
    }
    SetChanCode = (DLL_SET_CHAN_CODE)GetProcAddress(hLibM203, "SetChanCode");
    if(!SetChanCode)
    {
        MessageBox(0, "failed to find function:\n     SetChanCode(...)"
                "\nin M201_98.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		FreeLibrary(hLibM203);
        return FALSE;
    }
    StartScanning = (DLL_START_SCANNING)GetProcAddress(hLibM203, "StartScanning");
    if(!StartScanning)
    {
        MessageBox(0, "failed to find function:\n     StartScanning(...)"
                "\nin M201_98.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		FreeLibrary(hLibM203);
        return FALSE;
    }
    GetOneScan = (DLL_GET_ONE_SCAN)GetProcAddress(hLibM203, "GetOneScan");
    if(!GetOneScan)
    {
        MessageBox(0, "failed to find function:\n     GetOneScan(...)"
                "\nin M201_98.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		FreeLibrary(hLibM203);
        return FALSE;
    }
    StopScanning = (DLL_STOP_SCANNING)GetProcAddress(hLibM203, "StopScanning");
    if(!StopScanning)
    {
        MessageBox(0, "failed to find function:\n     StopScanning(...)"
                "\nin M201_98.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		FreeLibrary(hLibM203);
        return FALSE;
    }
    RemFromBuffer = (DLL_REM_FROM_BUFFER)GetProcAddress(hLibM203, "RemFromBuffer");
    if(!RemFromBuffer)
    {
        MessageBox(0, "failed to find function:\n     RemFromBuffer(...)"
                "\nin M201_98.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		FreeLibrary(hLibM203);
        return FALSE;
    }
    SendADMode = (DLL_SEND_AD_MODE)GetProcAddress(hLibM203, "SendADMode");
    if(!SendADMode)
    {
        MessageBox(0, "failed to find function:\n     SendADMode(...)"
                "\nin M201_98.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		FreeLibrary(hLibM203);
        return FALSE;
    }
    GetCheckSum = (DLL_GET_CHECKSUM)GetProcAddress(hLibM203, "GetCheckSum");
    if(!GetCheckSum)
    {
        MessageBox(0, "failed to find function:\n     GetCheckSum(...)"
                "\nin M201_98.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		FreeLibrary(hLibM203);
        return FALSE;
    }
    CloseDevice = (DLL_CLOSE_DEVICE)GetProcAddress(hLibM203, "CloseDevice");
    if(!CloseDevice)
    {
        MessageBox(0, "failed to find function:\n     CloseDevice(...)"
                "\nin M201_98.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		FreeLibrary(hLibM203);
        return FALSE;
    }
    CheckScanTime = (DLL_CHECK_SCAN_TIME)GetProcAddress(hLibM203, "CheckScanTime");
    if(!CheckScanTime)
    {
        MessageBox(0, "failed to find function:\n     CheckScanTime(...)"
                "\nin M201_98.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		FreeLibrary(hLibM203);
        return FALSE;
    }
    ChangeAnalogOutput = (DLL_CHANGE_ANALOG_OUTPUT)
					GetProcAddress(hLibM203, "ChangeAnalogOutput");
    if(!ChangeAnalogOutput)
    {
        MessageBox(0, "failed to find function:\n     ChangeAnalogOutput(...)"
                "\nin M201_98.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		FreeLibrary(hLibM203);
        return FALSE;
    }
    ClearVoltageArray = (DLL_CLEAR_VOLTAGE_ARRAY)
					GetProcAddress(hLibM203, "ClearVoltageArray");
    if(!ClearVoltageArray)
    {
        MessageBox(0, "failed to find function:\n     ClearVoltageArray(...)"
                "\nin M201_98.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		FreeLibrary(hLibM203);
        return FALSE;
    }

    GetNumScansInBuffer = (DLL_GET_NUM_SCANS_IN_BUFFER)
					GetProcAddress(hLibM203, "GetNumScansInBuffer");
    if(!GetNumScansInBuffer)
    {
        MessageBox(0, "failed to find function:\n     GetNumScansInBuffer(...)"
                "\nin M201_98.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		FreeLibrary(hLibM203);
        return FALSE;
    }
    Gconv = (DLL_GCONV)
					GetProcAddress(hLibM203, "Gconv");
    if(!Gconv)
    {
        MessageBox(0, "failed to find function:\n     Gconv(...)"
                "\nin M201_98.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		FreeLibrary(hLibM203);
        return FALSE;
    }

    GetVoltage = (DLL_GET_VOLTAGE)
					GetProcAddress(hLibM203, "GetVoltage");
    if(!GetVoltage)
    {
        MessageBox(0, "failed to find function:\n     GetVoltage(...)"
                "\nin M201_98.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		FreeLibrary(hLibM203);
        return FALSE;
    }



	return Status;
}

